@@ -1,4 +1,4 @@
-/* $Id: udns.h,v 1.37 2005/09/12 12:09:10 mjt Exp $
+/* udns.h
    header file for the UDNS library.
 
    Copyright (C) 2005  Michael Tokarev <mjt@corpit.ru>
@@ -23,9 +23,9 @@
 
 #ifndef UDNS_VERSION	/* include guard */
 
-#define UDNS_VERSION "0.0.8"
+#define UDNS_VERSION "0.2"
 
-#ifdef WIN32
+#ifdef WINDOWS
 # ifdef UDNS_DYNAMIC_LIBRARY
 #  ifdef DNS_LIBRARY_BUILD
 #   define UDNS_API __declspec(dllexport)
@@ -69,12 +69,12 @@ typedef unsigned char dnsc_t;
 typedef const unsigned char dnscc_t;
 
 #define DNS_MAXDN	255	/* max DN length */
+#define DNS_DNPAD	1	/* padding for DN buffers */
 #define DNS_MAXLABEL	63	/* max DN label length */
 #define DNS_MAXNAME	1024	/* max asciiz domain name length */
 #define DNS_HSIZE	12	/* DNS packet header size */
 #define DNS_PORT	53	/* default domain port */
 #define DNS_MAXSERV	6	/* max servers to consult */
-#define DNS_MAXSRCH	5	/* max searchlist entries */
 #define DNS_MAXPACKET	512	/* max traditional-DNS UDP packet size */
 #define DNS_EDNS0PACKET	4096	/* EDNS0 packet size to use */
 
@@ -130,7 +130,20 @@ enum dns_type {		/* DNS RR Types */
   DNS_T_SINK		= 40,	/* Kitchen sink (experimentatl) */
   DNS_T_OPT		= 41,	/* EDNS0 option (meta-RR) */
   DNS_T_DS		= 43,	/* DNSSEC */
+  DNS_T_SSHFP		= 44,
+  DNS_T_IPSECKEY	= 45,
+  DNS_T_RRSIG		= 46,	/* DNSSEC */
   DNS_T_NSEC		= 47,	/* DNSSEC */
+  DNS_T_DNSKEY		= 48,
+  DNS_T_DHCID		= 49,
+  DNS_T_NSEC3		= 50,
+  DNS_T_NSEC3PARAMS	= 51,
+  DNS_T_TALINK		= 58, /* draft-ietf-dnsop-trust-history */
+  DNS_T_SPF		= 99,
+  DNS_T_UINFO		= 100,
+  DNS_T_UID		= 101,
+  DNS_T_GID		= 102,
+  DNS_T_UNSPEC		= 103,
   DNS_T_TSIG		= 250,	/* Transaction signature. */
   DNS_T_IXFR		= 251,	/* Incremental zone transfer. */
   DNS_T_AXFR		= 252,	/* Transfer zone of authority. */
@@ -138,6 +151,7 @@ enum dns_type {		/* DNS RR Types */
   DNS_T_MAILA		= 254,	/* Transfer mail agent records. */
   DNS_T_ANY		= 255,	/* Wildcard match. */
   DNS_T_ZXFR		= 256,	/* BIND-specific, nonstandard. */
+  DNS_T_DLV		= 32769, /* RFC 4431, 5074, DNSSEC Lookaside Validation */
   DNS_T_MAX		= 65536
 };
 
@@ -206,6 +220,10 @@ dns_dntosp(dnscc_t *dn);
 UDNS_API unsigned
 dns_dntop_size(dnscc_t *dn);
 
+/* either wrappers or reimplementations for inet_ntop() and inet_pton() */
+UDNS_API const char *dns_ntop(int af, const void *src, char *dst, int size);
+UDNS_API int dns_pton(int af, const char *src, void *dst);
+
 /**************************************************************************/
 /**************** DNS raw packet layout ***********************************/
 
@@ -237,11 +255,11 @@ static __inline unsigned dns_get32(dnscc_t *s) {
         | ((unsigned)s[2]<<8) | s[3];
 }
 static __inline dnsc_t *dns_put16(dnsc_t *d, unsigned n) {
-  *d++ = (n >> 8) & 255; *d++ = n & 255; return d;
+  *d++ = (dnsc_t)((n >> 8) & 255); *d++ = (dnsc_t)(n & 255); return d;
 }
 static __inline dnsc_t *dns_put32(dnsc_t *d, unsigned n) {
-  *d++ = (n >> 24) & 255; *d++ = (n >> 16) & 255;
-  *d++ = (n >>  8) & 255; *d++ = n & 255;
+  *d++ = (dnsc_t)((n >> 24) & 255); *d++ = (dnsc_t)((n >> 16) & 255);
+  *d++ = (dnsc_t)((n >>  8) & 255); *d++ = (dnsc_t)(n & 255);
   return d;
 }
 
@@ -268,7 +286,11 @@ enum {
   DNS_H_F2	= 3,
   DNS_HF2_RA	= 0x80,	/* recursion available */
 #define dns_ra(pkt)	((pkt)[DNS_H_F2]&DNS_HF2_RA)
-  DNS_HF2_Z	= 0x70,	/* reserved */
+  DNS_HF2_Z	= 0x40,	/* reserved */
+  DNS_HF2_AD	= 0x20, /* DNSSEC: authentic data */
+#define dns_ad(pkt)	((pkt)[DNS_H_F2]&DNS_HF2_AD)
+  DNS_HF2_CD	= 0x10, /* DNSSEC: checking disabled */
+#define dns_cd(pkt)	((pkt)[DNS_H_F2]&DNS_HF2_CD)
   DNS_HF2_RCODE	= 0x0f,	/* response code, DNS_R_XXX above */
 #define dns_rcode(pkt)	((pkt)[DNS_H_F2]&DNS_HF2_RCODE)
  /* bytes 4:5: qdcount, numqueries */
@@ -292,6 +314,8 @@ enum {
   DNS_H_ARCNT	= DNS_H_ARCNT1,
 #define dns_numar(pkt)	dns_get16((pkt)+10)
 #define dns_payload(pkt) ((pkt)+DNS_HSIZE)
+  /* EDNS0 (OPT RR) flags (Ext. Flags) */
+  DNS_EF1_DO	= 0x80, /* DNSSEC OK */
 };
 
 /* packet buffer: start at pkt, end before pkte, current pos *curp.
@@ -350,16 +374,19 @@ dns_rewind(struct dns_parse *p, dnscc_t *qdn);
 /* default resolver context */
 UDNS_DATA_API extern struct dns_ctx dns_defctx;
 
-/* initialize default resolver context and open it if do_open is true.
- * <0 on failure. */
+/* reset resolver context to default state, close it if open, drop queries */
+UDNS_API void
+dns_reset(struct dns_ctx *ctx);
+
+/* reset resolver context and read in system configuration */
 UDNS_API int
-dns_init(int do_open);
+dns_init(struct dns_ctx *ctx, int do_open);
 
 /* return new resolver context with the same settings as copy */
 UDNS_API struct dns_ctx *
 dns_new(const struct dns_ctx *copy);
 
-/* free resolver context; all queries are dropped */
+/* free resolver context returned by dns_new(); all queries are dropped */
 UDNS_API void
 dns_free(struct dns_ctx *ctx);
 
@@ -367,7 +394,7 @@ dns_free(struct dns_ctx *ctx);
 UDNS_API int
 dns_add_serv(struct dns_ctx *ctx, const char *serv);
 
-/* add nameserver using struct sockaddr array (with ports etc) */
+/* add nameserver using struct sockaddr structure (with ports) */
 UDNS_API int
 dns_add_serv_s(struct dns_ctx *ctx, const struct sockaddr *sa);
 
@@ -396,7 +423,8 @@ enum dns_flags {
   DNS_NOSRCH	= 0x00010000,	/* do not perform search */
   DNS_NORD	= 0x00020000,	/* request no recursion */
   DNS_AAONLY	= 0x00040000,	/* set AA flag in queries */
-  DNS_PASSALL	= 0x00080000,	/* pass all replies to application */
+  DNS_SET_DO	= 0x00080000,	/* set EDNS0 "DO" bit (DNSSEC OK) */
+  DNS_SET_CD	= 0x00100000,	/* set CD bit (DNSSEC: checking disabled) */
 };
 
 /* set the debug function pointer */
@@ -479,8 +507,7 @@ dns_submit_p(struct dns_ctx *ctx,
 UDNS_API int
 dns_cancel(struct dns_ctx *ctx, struct dns_query *q);
 
-/* immediately resolve a generic query, return the answer
- * and place completion status into *statusp */
+/* resolve a generic query, return the answer */
 UDNS_API void *
 dns_resolve_dn(struct dns_ctx *ctx,
                dnscc_t *qdn, int qcls, int qtyp, int flags,
@@ -636,6 +663,32 @@ dns_resolve_srv(struct dns_ctx *ctx,
                 const char *name, const char *srv, const char *proto,
                 int flags);
 
+/* NAPTR (RFC3403) RR type */
+struct dns_naptr {	/* single NAPTR RR */
+  int order;		/* NAPTR order */
+  int preference;	/* NAPTR preference */
+  char *flags;		/* NAPTR flags */
+  char *service;	/* NAPTR service */
+  char *regexp;		/* NAPTR regexp */
+  char *replacement;	/* NAPTR replacement */
+};
+
+struct dns_rr_naptr {		/* the NAPTR RRset */
+  dns_rr_common(dnsnaptr);
+  struct dns_naptr *dnsnaptr_naptr;	/* array of NAPTRes */
+};
+UDNS_API dns_parse_fn dns_parse_naptr;	/* NAPTR RR parsing routine */
+typedef void				/* NAPTR RR callback */
+dns_query_naptr_fn(struct dns_ctx *ctx,
+                   struct dns_rr_naptr *result, void *data);
+/* submit NAPTR IN query */
+UDNS_API struct dns_query *
+dns_submit_naptr(struct dns_ctx *ctx, const char *name, int flags,
+                 dns_query_naptr_fn *cbck, void *data);
+/* resolve NAPTR IN query */
+UDNS_API struct dns_rr_naptr *
+dns_resolve_naptr(struct dns_ctx *ctx, const char *name, int flags);
+
 
 UDNS_API struct dns_query *
 dns_submit_a4dnsbl(struct dns_ctx *ctx,
@@ -693,17 +746,31 @@ UDNS_DATA_API extern const struct dns_nameval dns_typetab[];
 UDNS_DATA_API extern const struct dns_nameval dns_rcodetab[];
 UDNS_API int
 dns_findname(const struct dns_nameval *nv, const char *name);
-#define dns_findclassname(class) dns_findname(dns_classtab, (class))
+#define dns_findclassname(cls) dns_findname(dns_classtab, (cls))
 #define dns_findtypename(type) dns_findname(dns_typetab, (type))
 #define dns_findrcodename(rcode) dns_findname(dns_rcodetab, (rcode))
 
-UDNS_API const char *dns_classname(enum dns_class class);
+UDNS_API const char *dns_classname(enum dns_class cls);
 UDNS_API const char *dns_typename(enum dns_type type);
 UDNS_API const char *dns_rcodename(enum dns_rcode rcode);
 const char *_dns_format_code(char *buf, const char *prefix, int code);
 
 UDNS_API const char *dns_strerror(int errnum);
 
+/* simple pseudo-random number generator, code by Bob Jenkins */
+
+struct udns_jranctx {	/* the context */
+  unsigned a, b, c, d;
+};
+
+/* initialize the RNG with a given seed */
+UDNS_API void
+udns_jraninit(struct udns_jranctx *x, unsigned seed);
+
+/* return next random number.  32bits on most platforms so far. */
+UDNS_API unsigned
+udns_jranval(struct udns_jranctx *x);
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
